==> kazehakase-0.0.3.tar.gz: kazehakase-0.0.3/src/kz-mozembed.c <==
/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2002 Hiroyuki Ikezoe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "kz-mozembed.h"

#include <string.h>
#include <gtkmozembed.h>

#include "gobject-utils.h"
#include "mozilla.h"
#include "prefs.h"

static void kz_moz_embed_class_init    (KzMozEmbedClass *class);
static void kz_moz_embed_init          (KzMozEmbed *kzembed);
static void kz_moz_embed_destroy       (GtkObject *object);
static void kz_moz_embed_title         (GtkMozEmbed *embed);
static void kz_moz_embed_location      (GtkMozEmbed *embed);
static void kz_moz_embed_net_start     (GtkMozEmbed *embed);
static gint kz_moz_embed_open_uri      (GtkMozEmbed *embed, const char *uri);
static void kz_moz_embed_size_to       (GtkMozEmbed *embed,
					gint width, gint height);
static void kz_moz_embed_net_stop      (GtkMozEmbed *embed);
static gint kz_moz_embed_dom_key_down  (GtkMozEmbed *embed, gpointer event);
static gint kz_moz_embed_dom_key_up    (GtkMozEmbed *embed, gpointer event);
static gint kz_moz_embed_dom_key_press (GtkMozEmbed *embed, gpointer event);

static GtkMozEmbedClass *parent_class = NULL;

KZ_OBJECT_GET_TYPE(kz_moz_embed, "KzMozEmbed", KzMozEmbed,
		   kz_moz_embed_class_init, kz_moz_embed_init,
		   GTK_TYPE_MOZ_EMBED)

static void
kz_moz_embed_class_init (KzMozEmbedClass *class)
{
	GtkObjectClass *object_class;
	GtkMozEmbedClass *moz_embed_class;

	parent_class = g_type_class_peek_parent (class);

	object_class = (GtkObjectClass *) class;
	moz_embed_class = (GtkMozEmbedClass *) class;

	/* GtkObject signals */
	object_class->destroy = kz_moz_embed_destroy;

	/* GtkMozEmbedSignals */
#if 0
	moz_embed_class->dom_mouse_over  = kz_moz_embed_dom_mouse_over;
	moz_embed_class->dom_mouse_out   = kz_moz_embed_dom_mouse_out;
	moz_embed_class->visibility      = kz_moz_embed_visibility;
	moz_embed_class->net_state       = kz_moz_embed_net_state;
	moz_embed_class->net_state_all   = kz_moz_embed_net_state_all;
	moz_embed_class->progress        = kz_moz_embed_progress;
	moz_embed_class->progress_all    = kz_moz_embed_progress_all;
	moz_embed_class->js_status       = kz_moz_embed_js_status;
	moz_embed_class->destroy_browser = kz_moz_embed_destroy_browser;
#endif

	moz_embed_class->title           = kz_moz_embed_title;
	moz_embed_class->location        = kz_moz_embed_location;
	moz_embed_class->net_start       = kz_moz_embed_net_start;
	moz_embed_class->net_stop        = kz_moz_embed_net_stop;
	moz_embed_class->open_uri        = kz_moz_embed_open_uri;
	moz_embed_class->size_to         = kz_moz_embed_size_to;
	moz_embed_class->dom_key_press   = kz_moz_embed_dom_key_press;
	moz_embed_class->dom_key_down    = kz_moz_embed_dom_key_down;
	moz_embed_class->dom_key_up      = kz_moz_embed_dom_key_up;
}

static void
kz_moz_embed_init (KzMozEmbed *kzembed)
{
	/* widgets */
	kzembed->parent_kazehakase     = NULL;
	kzembed->popup_window          = NULL;

	/* status */
	kzembed->location              = NULL;
	kzembed->title                 = NULL;
	kzembed->load_started          = 0;
	kzembed->load_percent          = 0;
	kzembed->bytes_loaded          = 0;
	kzembed->max_bytes_loaded      = 0;
	kzembed->is_loading            = FALSE;
	kzembed->load_status_message   = NULL;

	/* gesture */
	kzembed->gesture               = kz_gesture_new();
	kzembed->gesture_location      = NULL;
}

GtkWidget *
kz_moz_embed_new (KzWindow *kz, const gchar *url)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(g_object_new (KZ_TYPE_MOZ_EMBED, NULL));

	kzembed->parent_kazehakase = kz;

#if 0
	/*
	 * set the chrome type so it's stored in the object
	 */
	g_moz_embed_set_chrome_mask(GTK_MOZ_EMBED(kzembed->mozembed),
				    actualChromeMask);
#endif

	gtk_moz_embed_load_url(GTK_MOZ_EMBED(kzembed), url);

	return GTK_WIDGET (kzembed);
}

static void
kz_moz_embed_destroy (GtkObject *object)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(object);

	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));

	g_free (kzembed->location);
	kzembed->location = NULL;
	g_free (kzembed->title);
	kzembed->title = NULL;
	g_free (kzembed->gesture_location);
	kzembed->gesture_location = NULL;

	if (GTK_OBJECT_CLASS (parent_class)->destroy)
		GTK_OBJECT_CLASS (parent_class)->destroy(object);
}

const gchar *
kz_moz_embed_get_title (KzMozEmbed *kzembed)
{
	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), NULL);
	return kzembed->title;
}

const gchar *
kz_moz_embed_get_location (KzMozEmbed *kzembed)
{
	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), NULL);
	return kzembed->location;
}

static void
kz_moz_embed_title (GtkMozEmbed *embed)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(embed);

	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));

	g_free(kzembed->title);
	kzembed->title = mozilla_get_title(embed);
}

static void
kz_moz_embed_location (GtkMozEmbed *embed)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(embed);

	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));

	g_free(kzembed->location);
	kzembed->location = gtk_moz_embed_get_location(embed);
}

static void
kz_moz_embed_net_start (GtkMozEmbed *embed)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(embed);

	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));

	if (parent_class->net_start)
		parent_class->net_start(embed);

	kzembed->is_loading = TRUE;
}

static void
kz_moz_embed_net_stop (GtkMozEmbed *embed)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(embed);

	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));

	if (parent_class->net_stop)
		parent_class->net_stop(embed);

	kzembed->is_loading = FALSE;
}

static gint
kz_moz_embed_open_uri (GtkMozEmbed *embed, const char *uri)
{
	if (parent_class->open_uri)
		parent_class->open_uri(embed, uri);

	if (!strncmp(uri, "mailto:", 7))
	{
		return TRUE;
	}

	return FALSE;
}

static void
kz_moz_embed_size_to (GtkMozEmbed *embed, gint width, gint height)
{
	gtk_widget_set_size_request(GTK_WIDGET(embed), width, height);
}

static gint
kz_moz_embed_dom_key_down (GtkMozEmbed *embed, gpointer event)
{
#if 0
	gulong key_code;
	
	mozilla_get_key_code(event, &key_code);
	printf("down key=%d\n",key_code);
#endif

	return FALSE;
}

static gint
kz_moz_embed_dom_key_up (GtkMozEmbed *embed, gpointer event)
{
#if 0
	gulong key_code;
	KzWindow *kz = kzembed->parent_kazehakase;
     	glong type;
	KeyEventInfo info;

	mozilla_get_key_code(event, &key_code);

     	type = mozilla_get_key_event_info(embed, event,&info);

	if ((key_code == 191) && !(type & CONTEXT_INPUT)) /* '/' */
	{
		gtk_widget_grab_focus(kz->findArea);
	}
#endif

	return FALSE;
}

static gint
kz_moz_embed_dom_key_press (GtkMozEmbed *embed, gpointer event)
{
#if 0
	gulong char_code;
	
	mozilla_get_char_code(event, &char_code);
	printf("press key=%d\n",char_code);
#endif

	return FALSE;
}

==> kazehakase-0.0.4.tar.gz: kazehakase-0.0.4/src/kz-mozembed.c <==
/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2002 Hiroyuki Ikezoe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "kz-mozembed.h"

#include <string.h>
#include <gtkmozembed.h>

#include "gobject-utils.h"
#include "mozilla.h"
#include "prefs.h"

static void kz_moz_embed_class_init    (KzMozEmbedClass *class);
static void kz_moz_embed_init          (KzMozEmbed *kzembed);
static void kz_moz_embed_destroy       (GtkObject *object);
static void kz_moz_embed_title         (GtkMozEmbed *embed);
static void kz_moz_embed_location      (GtkMozEmbed *embed);
static void kz_moz_embed_net_start     (GtkMozEmbed *embed);
static gint kz_moz_embed_open_uri      (GtkMozEmbed *embed, const char *uri);
static void kz_moz_embed_size_to       (GtkMozEmbed *embed,
					gint width, gint height);
static void kz_moz_embed_visibility    (GtkMozEmbed *embed,
					gboolean visibility);
static void kz_moz_embed_net_stop      (GtkMozEmbed *embed);
static gint kz_moz_embed_dom_key_down  (GtkMozEmbed *embed, gpointer event);
static gint kz_moz_embed_dom_key_up    (GtkMozEmbed *embed, gpointer event);
static gint kz_moz_embed_dom_key_press (GtkMozEmbed *embed, gpointer event);

void mozilla_setup();

static GtkMozEmbedClass *parent_class = NULL;

KZ_OBJECT_GET_TYPE(kz_moz_embed, "KzMozEmbed", KzMozEmbed,
		   kz_moz_embed_class_init, kz_moz_embed_init,
		   GTK_TYPE_MOZ_EMBED)

static void
kz_moz_embed_class_init (KzMozEmbedClass *class)
{
	GtkObjectClass *object_class;
	GtkMozEmbedClass *moz_embed_class;

	mozilla_setup();

	parent_class = g_type_class_peek_parent (class);

	object_class = (GtkObjectClass *) class;
	moz_embed_class = (GtkMozEmbedClass *) class;

	/* GtkObject signals */
	object_class->destroy = kz_moz_embed_destroy;

	/* GtkMozEmbedSignals */
#if 0
	moz_embed_class->dom_mouse_over  = kz_moz_embed_dom_mouse_over;
	moz_embed_class->dom_mouse_out   = kz_moz_embed_dom_mouse_out;
	moz_embed_class->net_state       = kz_moz_embed_net_state;
	moz_embed_class->net_state_all   = kz_moz_embed_net_state_all;
	moz_embed_class->progress        = kz_moz_embed_progress;
	moz_embed_class->progress_all    = kz_moz_embed_progress_all;
	moz_embed_class->js_status       = kz_moz_embed_js_status;
	moz_embed_class->destroy_browser = kz_moz_embed_destroy_browser;
	moz_embed_class->size_to         = kz_moz_embed_size_to;
#endif

	moz_embed_class->title           = kz_moz_embed_title;
	moz_embed_class->location        = kz_moz_embed_location;
	moz_embed_class->net_start       = kz_moz_embed_net_start;
	moz_embed_class->net_stop        = kz_moz_embed_net_stop;
	moz_embed_class->open_uri        = kz_moz_embed_open_uri;
	moz_embed_class->visibility      = kz_moz_embed_visibility;
	moz_embed_class->dom_key_press   = kz_moz_embed_dom_key_press;
	moz_embed_class->dom_key_down    = kz_moz_embed_dom_key_down;
	moz_embed_class->dom_key_up      = kz_moz_embed_dom_key_up;
}

static void
kz_moz_embed_init (KzMozEmbed *kzembed)
{
	/* widgets */
	kzembed->parent_kazehakase     = NULL;
	kzembed->popup_window          = NULL;

	/* status */
	kzembed->location              = NULL;
	kzembed->title                 = NULL;
	kzembed->load_started          = 0;
	kzembed->load_percent          = 0;
	kzembed->bytes_loaded          = 0;
	kzembed->max_bytes_loaded      = 0;
	kzembed->is_loading            = FALSE;
	kzembed->load_status_message   = NULL;

	/* gesture */
	kzembed->gesture               = kz_gesture_new();
	kzembed->gesture_location      = NULL;
}

GtkWidget *
kz_moz_embed_new (KzWindow *kz, const gchar *url)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(g_object_new (KZ_TYPE_MOZ_EMBED, NULL));

	kzembed->parent_kazehakase = kz;
#if 0
	/*
	 * set the chrome type so it's stored in the object
	 */
	g_moz_embed_set_chrome_mask(GTK_MOZ_EMBED(kzembed->mozembed),
				    actualChromeMask);
#endif
	gtk_moz_embed_load_url(GTK_MOZ_EMBED(kzembed), url);

	return GTK_WIDGET (kzembed);
}

static void
kz_moz_embed_destroy (GtkObject *object)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(object);

	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));

	g_free (kzembed->location);
	kzembed->location = NULL;
	g_free (kzembed->title);
	kzembed->title = NULL;
	if (kzembed->gesture)
		g_object_unref(kzembed->gesture);
	kzembed->gesture = NULL;
	g_free (kzembed->gesture_location);
	kzembed->gesture_location = NULL;

	if (GTK_OBJECT_CLASS (parent_class)->destroy)
		GTK_OBJECT_CLASS (parent_class)->destroy(object);
}

const gchar *
kz_moz_embed_get_title (KzMozEmbed *kzembed)
{
	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), NULL);
	return kzembed->title;
}

const gchar *
kz_moz_embed_get_location (KzMozEmbed *kzembed)
{
	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), NULL);
	return kzembed->location;
}

static void
kz_moz_embed_title (GtkMozEmbed *embed)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(embed);

	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));

	g_free(kzembed->title);
	kzembed->title = mozilla_get_title(embed);
}

static void
kz_moz_embed_location (GtkMozEmbed *embed)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(embed);

	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));

	g_free(kzembed->location);
	kzembed->location = gtk_moz_embed_get_location(embed);
}

static void
kz_moz_embed_net_start (GtkMozEmbed *embed)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(embed);

	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));

	if (parent_class->net_start)
		parent_class->net_start(embed);

	kzembed->is_loading = TRUE;
}

static void
kz_moz_embed_net_stop (GtkMozEmbed *embed)
{
	KzMozEmbed *kzembed = KZ_MOZ_EMBED(embed);

	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));

	if (parent_class->net_stop)
		parent_class->net_stop(embed);

	kzembed->is_loading = FALSE;
}

static gint
kz_moz_embed_open_uri (GtkMozEmbed *embed, const char *uri)
{
	if (parent_class->open_uri)
		parent_class->open_uri(embed, uri);

	if (!strncmp(uri, "mailto:", 7))
	{
		return TRUE;
	}

	return FALSE;
}

#if 0
static void
kz_moz_embed_size_to (GtkMozEmbed *embed, gint width, gint height)
{
	gtk_widget_set_size_request(GTK_WIDGET(embed), width, height);
}
#endif

static gint
kz_moz_embed_dom_key_down (GtkMozEmbed *embed, gpointer event)
{
	return FALSE;
}

static gint
kz_moz_embed_dom_key_up (GtkMozEmbed *embed, gpointer event)
{
	if (!prefs.typeahead)
	{
		KzMozEmbed *kzembed = KZ_MOZ_EMBED(embed);
		g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), FALSE);

		KzWindow *kz = KZ_MOZ_EMBED(kzembed)->parent_kazehakase;
		gulong key_code;
		glong type;
		KeyEventInfo info;
		
		mozilla_get_key_code(event, &key_code);
		
		type = mozilla_get_key_event_info(embed, event,&info);

		if ((key_code == 191) && !(type & CONTEXT_INPUT)) /* '/' */
		{
			gtk_widget_grab_focus(kz->findArea);
		}
	}

	return FALSE;
}

static gint
kz_moz_embed_dom_key_press (GtkMozEmbed *embed, gpointer event)
{
	return FALSE;
}

static void
kz_moz_embed_visibility (GtkMozEmbed *embed, gboolean visibility)
{
	GtkWidget *parent = NULL;
	parent = gtk_widget_get_parent(GTK_WIDGET(embed));
	g_return_if_fail(parent != NULL);
	
	if (visibility) 
	{
		gtk_widget_show(GTK_WIDGET(embed));
		gtk_widget_show(parent);
	}
	else
	{
		gtk_widget_hide(GTK_WIDGET(embed));
		gtk_widget_hide(parent);
	}
}

static void 
destroy_browser_cb (GtkMozEmbed *embed, GtkWidget *transient_window)
{
	gtk_widget_destroy(GTK_WIDGET(transient_window));
}

static void
new_window_orphan_cb (GtkMozEmbedSingle *embed, GtkMozEmbed **retval,
		      guint chromemask, gpointer data)
{
	/* FIXME! but I don't have any nice idea... */
	GtkWidget *newWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	GtkMozEmbed *newBrowser = GTK_MOZ_EMBED(kz_moz_embed_new(NULL, NULL));
	gtk_container_add(GTK_CONTAINER(newWindow), GTK_WIDGET(newBrowser));
	g_signal_connect(G_OBJECT(newBrowser),"destroy_browser",
			 G_CALLBACK(destroy_browser_cb),newWindow);
	*retval = GTK_MOZ_EMBED(newBrowser);
}

static gboolean
mozilla_prefs_init()
{
	const char *home_path;
	char full_path[256];
	
     	home_path = g_get_home_dir();
	if (!home_path) {
		g_error("Failed to get HOME");
		return FALSE;
	}

	g_snprintf(full_path, G_N_ELEMENTS(full_path),
		   "%s/%s", home_path, ".kazehakase/mozilla");
	gtk_moz_embed_set_profile_path(full_path, "kazehakase");

	return TRUE;
}

static void 
mozilla_prefs_set()
{
	/* set user agent name */
	mozilla_preference_set ("general.useragent.override",
				prefs.user_agent);
	/* set typeahead or not  */
	mozilla_preference_set_boolean ("accessibility.typeaheadfind",
				prefs.typeahead);	
}

void
mozilla_setup()
{
	GtkMozEmbedSingle *single = NULL;
	
	mozilla_prefs_init();
	
	gtk_moz_embed_push_startup();

	mozilla_prefs_set();
	
	/*
	 * get the singleton object and hook up to its new window callback
	 * so we can create orphaned windows.
	 */

	single = gtk_moz_embed_single_get();
	if (!single) {
		g_error("Failed to get singleton embed object!");
		return;
	}

	g_signal_connect(G_OBJECT(single), "new_window_orphan",
			 G_CALLBACK(new_window_orphan_cb), NULL);
}
